import { FormControl, FormGroup } from '@angular/forms';
import { __spread, __awaiter, __generator, __values, __assign } from 'tslib';
import { AsyncSubject, Observable, Subject, ReplaySubject, BehaviorSubject, concat, of } from 'rxjs';
import stringify from 'fast-json-stable-stringify';
import { filter, map, take, tap, debounceTime } from 'rxjs/operators';
import { ɵɵdefineInjectable, ɵsetClassMetadata, Injectable, ɵɵdefineDirective, ɵɵNgOnChangesFeature, ɵɵdefinePipe, Pipe, ɵɵdirectiveInject, ElementRef, ɵɵclassProp, Directive, HostBinding, Input, ɵɵdefineNgModule, ɵɵdefineInjector, ɵɵsetNgModuleScope, NgModule, InjectionToken, ɵɵinject, Inject, Optional } from '@angular/core';

function callFuncOnFormGroupAndChildren(formGroup, nameOfFunc) {
    var _this = this;
    if (!formGroup.controls)
        return;
    Object.keys(formGroup.controls)
        .map(function (formControlName) { return formGroup.controls[formControlName]; })
        .forEach(function (control) {
        if (control instanceof FormControl) {
            control[nameOfFunc]();
        }
        else if (control instanceof FormGroup) {
            _this.callFuncOnFormGroupAndChildren(control);
        }
    });
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as touched
 * @param formGroup The form group to mark as touched
 */
function markFormGroupAsTouched(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsTouched');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as untouched
 * @param formGroup The form group to mark as untouched
 */
function markFormGroupAsUntouched(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsUntouched');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as dirty
 * @param formGroup The form group to mark as untouched
 */
function markFormGroupAsDirty(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsDirty');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as pristine
 * @param formGroup The form group to mark as pristine
 */
function markFormGroupAsPristine(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsPristine');
}
/**
 * marks the form and all it's children, including other formGroups or controls
 * as pending
 * @param formGroup The form group to mark as pending
 */
function markFormGroupAsPending(formGroup) {
    callFuncOnFormGroupAndChildren(formGroup, 'markAsPending');
}
/**
 * get all dirty controls inside a form
 * @param form The form in which to check for dirty controls
 */
function getDirtyControls(form) {
    return Object.keys(form.controls)
        .map(function (k) { return ({ control: form.controls[k], name: k }); })
        .filter(function (c) { return c.control.dirty; });
}
var FormHelper = /** @class */ (function () {
    function FormHelper() {
    }
    FormHelper.markFormGroupAsTouched = markFormGroupAsTouched;
    FormHelper.getDirtyControls = getDirtyControls;
    FormHelper.markFormGroupAsUntouched = markFormGroupAsUntouched;
    FormHelper.markFormGroupAsDirty = markFormGroupAsDirty;
    FormHelper.markFormGroupAsPristine = markFormGroupAsPristine;
    FormHelper.markFormGroupAsPending = markFormGroupAsPending;
    return FormHelper;
}());

/**
 * Escapes a html string to be safe, for example to show inside HTML
 * @param unsafe The HTML string which to escape
 */
function escapeXml(unsafeHTML) {
    return unsafeHTML.replace(/[<>&'"]/g, function (c) {
        switch (c) {
            case '<': return '&lt;';
            case '>': return '&gt;';
            case '&': return '&amp;';
            case '\'': return '&apos;';
            case '"': return '&quot;';
        }
    });
}
/**
 * Checks if a string is a number
 * @param ch The string which to check for number
 */
function isNumber(ch) {
    return !isNaN(parseInt(ch));
}
/**
 * Waits for a property on an object to be not undefined
 * And returns this property in an observable
 * @param obj The object which holds the property
 * @param (obj: TObject) => TProperty propertyAccessor the property accessor
 *   function
 * @param (prop: TProperty) => boolean the default checks if the property is
 *   not undefined to be valid and finish waiting. Here you can provide a
 *   custom function, e.g. also checking if it is not null
 * @param number retries How many 100 milliseconds it should wait for the
 *   property
 * @returns Observable<TProperty>
 */
function waitForProperty(obj, propertyAccessor, isPropertyValidFunc, retries) {
    if (isPropertyValidFunc === void 0) { isPropertyValidFunc = function (prop) { return prop !== undefined; }; }
    if (retries === void 0) { retries = 50; }
    var retry = 0;
    var returnSubject = new AsyncSubject();
    var interval = setInterval(function () {
        var property = propertyAccessor(obj);
        if (isPropertyValidFunc(property)) {
            clearInterval(interval);
            returnSubject.next(property);
            returnSubject.complete();
        }
        retry++;
        if (retry > retries) {
            clearInterval(interval);
            returnSubject.error('Property still undefined after ' + retries + ' retries');
            returnSubject.complete();
        }
    }, 100);
    return returnSubject.asObservable();
}
/**
 * Sets a css class on the document body if it does not already exists
 * @param cssClass The CSS class which to set on the document body
 */
function setBodyCssClass(cssClass) {
    if (document.body.className.indexOf(cssClass) === -1)
        document.body.className += ' ' + cssClass;
}
/**
 * Removes a css class from the document body
 * @param cssClass The CSS Class which to remove
 */
function removeBodyCssClass(cssClass) {
    document.body.className = document.body.className.replace(cssClass, '');
}
/**
 * Toggles a CSS class on the document body
 * @param cssClass The class which to toggle on the document body
 */
function toggleBodyCssClass(cssClass) {
    if (document.body.className.indexOf(cssClass) === -1)
        document.body.className += ' ' + cssClass;
    else
        document.body.className = document.body.className.replace(cssClass, '');
}
/**
 * Handy function if you want to check both null and undefined
 * @param value The value which to check for null or undefined
 */
function isNullOrUndefined(value) {
    return value === undefined || value === null;
}
/**
 * Checks if an object is empty, meaning it has to properties
 * @param obj The object which to check for empty
 */
function isObjectEmpty(obj) {
    if (obj === undefined || obj === null) {
        return true;
    }
    return Object.keys(obj).length === 0;
}
/**
 * Takes an object converts it to json and then URI-Encodes it to be used in a
 * URL
 * @param obj The object which to encode
 */
function encodeURIJson(obj) {
    var json = JSON.stringify(obj);
    return encodeURIComponent(json);
}
/**
 * Takes an URI value which is a json string and converts it to a javascript
 * object
 * @param encodedJson The URIEncoded JSON
 */
function decodeURIJson(encodedJson) {
    var json = decodeURIComponent(encodedJson);
    return JSON.parse(json);
}
/***
 * Clones an object via json stringify and json parse (so classes with
 * functions won't work here...) However plain objects and arrays are deep
 * cloned, this is why this method exists Also important: Because of deep
 * cloning objects in arrays are also cloned, so perfect for duplicating an
 * array without having the same reference inside Most useful also for
 * immutable Redux-State
 * @param obj The object to clone
*/
function jsonDeepClone(obj) {
    var json = JSON.stringify(obj);
    return JSON.parse(json);
}
/***
 * Compares 2 objects if they are equal by converting to json and comparing the
 * string So this method does not compare the reference but all the values
 * inside
 * @param obj1 First object to compare
 * @param obj2 Second object to compare
 */
function jsonIsEqual(obj1, obj2) {
    var json1 = stringify(obj1);
    var json2 = stringify(obj2);
    return json1 === json2;
}
/***
 * Used to preload image. Returns a stream containing the Html Image Element on
 * success or the error in case of an error
 * @param url The image src url to load
 */
function loadImage(url) {
    return new Observable(function (subscriber) {
        var image = new Image();
        image.onload = function (ev) {
            subscriber.next(image);
            subscriber.complete();
        };
        image.onerror = function (ev) {
            subscriber.next(ev.error);
            subscriber.complete();
        };
        image.src = url;
    });
}
/***
 * Javascript has no methode for rounding a number to certain amount of digits,
 * so here it is
 * @param num The number which to round
 * @param digits How many digits after the comma
 */
function roundNumber(num, digits) {
    if (digits === void 0) { digits = 2; }
    var digitOperator = '1' + '0'.repeat(digits);
    return Math.round(num * +digitOperator) / +digitOperator;
}
function createThumbnailFromBase64Image(base64Image, targetSize) {
    return new Observable(function (subscriber) {
        var img = new Image();
        img.onload = function () {
            var width = img.width;
            var height = img.height;
            var canvas = document.createElement('canvas');
            var ctx = canvas.getContext('2d');
            canvas.width = canvas.height = targetSize;
            ctx.drawImage(img, width > height ? (width - height) / 2 : 0, height > width ? (height - width) / 2 : 0, width > height ? height : width, width > height ? height : width, 0, 0, targetSize, targetSize);
            subscriber.next(canvas.toDataURL());
            subscriber.complete();
        };
        img.src = base64Image;
    });
}
function safeCall(obj, functionToCallGetterFuncOrName, defaultValue) {
    if (defaultValue === void 0) { defaultValue = undefined; }
    var params = [];
    for (var _i = 3; _i < arguments.length; _i++) {
        params[_i - 3] = arguments[_i];
    }
    if (!obj) {
        return defaultValue;
    }
    var functionToCall = typeof functionToCallGetterFuncOrName === 'string' ?
        obj[functionToCallGetterFuncOrName] : functionToCallGetterFuncOrName(obj);
    if (!functionToCall) {
        return defaultValue;
    }
    return functionToCall.call.apply(functionToCall, __spread([obj], params));
}
var MiscHelper = /** @class */ (function () {
    function MiscHelper() {
    }
    MiscHelper.safeCall = safeCall;
    MiscHelper.createThumbnailFromBase64Image = createThumbnailFromBase64Image;
    MiscHelper.escapeXml = escapeXml;
    MiscHelper.isNumber = isNumber;
    MiscHelper.waitForProperty = waitForProperty;
    MiscHelper.setBodyCssClass = setBodyCssClass;
    MiscHelper.removeBodyCssClass = removeBodyCssClass;
    MiscHelper.toggleBodyCssClass = toggleBodyCssClass;
    MiscHelper.isNullOrUndefined = isNullOrUndefined;
    MiscHelper.isObjectEmpty = isObjectEmpty;
    MiscHelper.encodeURIJson = encodeURIJson;
    MiscHelper.decodeURIJson = decodeURIJson;
    MiscHelper.jsonDeepClone = jsonDeepClone;
    MiscHelper.jsonIsEqual = jsonIsEqual;
    MiscHelper.roundNumber = roundNumber;
    MiscHelper.loadImage = loadImage;
    return MiscHelper;
}());

/**
 * Creates a hidden link to download something given the href url
 * @param url The url which to download
 * @param fileName The name of the file to be downloaded (used for the a tag)
 */
function createAndClickDownloadLink(url, fileName) {
    var link = document.createElement('a');
    if (link.download === undefined)
        return;
    link.setAttribute('href', url);
    link.setAttribute('download', fileName);
    link.style.visibility = 'hidden';
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
}
/**
 * Creates a csv data uri from an array, this uri can then be downloaded in the browser
 * using createAndClickDownloadLink
 * @param data an array of string arrays containing the csv data
 */
function createCsvDataUriFromArray(data, delimiter) {
    if (delimiter === void 0) { delimiter = ';'; }
    var csvContent = 'data:text/csv;charset=utf-8,';
    data.forEach(function (rowArray) {
        var row = rowArray.join(delimiter);
        csvContent += row + '\r\n';
    });
    return csvContent;
}
/**
 * Creates a csv file from the array using createCsvDataUriFromArray
 * And presents it to the user as a download file in the browser
 * @param data an array of string arrays containing the csv data
 * @param filename the name of the csv file which will be downloaded
 */
function downloadArrayAsCsvFile(data, filename, delimiter) {
    if (delimiter === void 0) { delimiter = ';'; }
    var csv = createCsvDataUriFromArray(data, delimiter);
    createAndClickDownloadLink(csv, filename);
}
/**
 * opens a file dialog to choose a file and returns the file object
 * including the data.
 * When it is an image the data is read as base64, so the dataType is 'base64'
 * All other files are read as binaryString
 * @param fileDialogAccept this is optional, choose the mime types of files
 * the user can select in the file dialog
 */
function getFile(fileDialogAccept) {
    if (fileDialogAccept === void 0) { fileDialogAccept = '*'; }
    return __awaiter(this, void 0, void 0, function () {
        var p;
        return __generator(this, function (_a) {
            p = new Promise(function (resolve, reject) {
                var element = document.createElement('div');
                element.innerHTML = "<input type=\"file\" accept=\"" + fileDialogAccept + "\">";
                var fileInput = element.firstChild;
                fileInput.addEventListener('change', function () {
                    var file = fileInput.files[0];
                    var reader = new FileReader();
                    var doResolve = function (data, dataType) {
                        element.remove();
                        resolve({
                            size: file.size,
                            lastModified: file.lastModified,
                            mimeType: file.type,
                            name: file.name,
                            dataType: dataType,
                            data: data
                        });
                    };
                    if (file.type.startsWith('image')) {
                        reader.onload = function () {
                            var s = reader.result;
                            var base64Part = s.substring(s.indexOf('base64,') + 7);
                            doResolve(base64Part, 'base64');
                        };
                        reader.readAsDataURL(file);
                    }
                    else {
                        reader.onload = function () {
                            doResolve(reader.result, 'binaryString');
                        };
                        reader.readAsBinaryString(file);
                    }
                });
                fileInput.click();
            });
            return [2 /*return*/, p];
        });
    });
}
var FileHelper = /** @class */ (function () {
    function FileHelper() {
    }
    FileHelper.getFile = getFile;
    FileHelper.createAndClickDownloadLink = createAndClickDownloadLink;
    FileHelper.createCsvDataUriFromArray = createCsvDataUriFromArray;
    FileHelper.downloadArrayAsCsvFile = downloadArrayAsCsvFile;
    return FileHelper;
}());

/**
 * Padding the string left
 * @param s the string which to pad
 * @param length the length the string should be padded to
 * @param padChar the char which to use to fill up the string
 */
function padLeft(s, length, padChar) {
    while (s.length < length)
        s = padChar + s;
    return s;
}
/**
 * Padding the string right
 * @param s the string which to pad
 * @param length the length the string should be padded to
 * @param padChar the char which to use to fill up the string
 */
function padRight(s, length, padChar) {
    while (s.length < length)
        s += padChar;
    return s;
}
/**
 * Replace all occurences in the string
 * @param str The string in which to replace something
 * @param find The string to replace
 * @param replace The string that will replace the find string
 */
function replaceAll(str, find, replace) {
    var e_1, _a;
    var ret = '';
    try {
        for (var str_1 = __values(str), str_1_1 = str_1.next(); !str_1_1.done; str_1_1 = str_1.next()) {
            var ch = str_1_1.value;
            if (ch === find) {
                ret += replace;
            }
            else {
                ret += ch;
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (str_1_1 && !str_1_1.done && (_a = str_1.return)) _a.call(str_1);
        }
        finally { if (e_1) throw e_1.error; }
    }
    return ret;
}
var StringHelper = /** @class */ (function () {
    function StringHelper() {
    }
    StringHelper.replaceAll = replaceAll;
    StringHelper.padLeft = padLeft;
    StringHelper.padRight = padRight;
    return StringHelper;
}());

function cSharpTicks2Date(ticks) {
    // ticks are in nanotime; convert to microtime
    var ticksToMicrotime = ticks / 10000;
    // ticks are recorded from 1/1/1; get microtime difference from 1/1/1/ to 1/1/1970
    var epochMicrotimeDiff = 62135600400000; // Math.abs(new Date(0, 0, 1, 0, 0, 0, 0).setFullYear(1));
    // new date is ticks, converted to microtime, minus difference from epoch microtime
    var tickDate = new Date(ticksToMicrotime - epochMicrotimeDiff);
    return tickDate;
}
function dateAddYears(date, years) {
    var result = new Date(date);
    result.setFullYear(result.getFullYear() + years);
    return result;
}
function dateSubstractYears(date, years) {
    var result = new Date(date);
    result.setFullYear(result.getFullYear() - years);
    return result;
}
function dateAddMonths(date, months) {
    var result = new Date(date);
    result.setMonth(result.getMonth() + months);
    return result;
}
function dateSubstractMonths(date, months) {
    var result = new Date(date);
    result.setMonth(result.getMonth() - months);
    return result;
}
function dateAddDays(date, days) {
    var result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
}
function dateSubstractDays(date, days) {
    var result = new Date(date);
    result.setDate(result.getDate() - days);
    return result;
}
function dateAddHours(date, hours) {
    var result = new Date(date);
    result.setHours(result.getHours() + hours);
    return result;
}
function dateSubstractHours(date, hours) {
    var result = new Date(date);
    result.setHours(result.getHours() - hours);
    return result;
}
function dateAddMinutes(date, minutes) {
    var result = new Date(date);
    result.setMinutes(result.getMinutes() + minutes);
    return result;
}
function dateSubstractMinutes(date, minutes) {
    var result = new Date(date);
    result.setMinutes(result.getMinutes() - minutes);
    return result;
}
function dateAddSeconds(date, seconds) {
    var result = new Date(date);
    result.setSeconds(result.getSeconds() + seconds);
    return result;
}
function dateSubstractSeconds(date, seconds) {
    var result = new Date(date);
    result.setSeconds(result.getSeconds() - seconds);
    return result;
}
function dateAddMilliSeconds(date, milliseconds) {
    var result = new Date(date);
    result.setMilliseconds(result.getMilliseconds() + milliseconds);
    return result;
}
function dateSubstractMilliSeconds(date, milliseconds) {
    var result = new Date(date);
    result.setMilliseconds(result.getMilliseconds() - milliseconds);
    return result;
}
function isToday(date) {
    var ret = true;
    var now = new Date(Date.now());
    if (now.getFullYear() !== date.getFullYear())
        ret = false;
    if (now.getMonth() !== date.getMonth())
        ret = false;
    if (now.getDate() !== date.getDate())
        ret = false;
    return ret;
}
function isYesterday(date) {
    var ret = true;
    var yesterday = dateSubstractDays(new Date(Date.now()), 1);
    if (yesterday.getFullYear() !== date.getFullYear())
        ret = false;
    if (yesterday.getMonth() !== date.getMonth())
        ret = false;
    if (yesterday.getDate() !== date.getDate())
        ret = false;
    return ret;
}
var IsNowGranulartiy;
(function (IsNowGranulartiy) {
    IsNowGranulartiy[IsNowGranulartiy["Hours"] = 0] = "Hours";
    IsNowGranulartiy[IsNowGranulartiy["Minutes"] = 1] = "Minutes";
    IsNowGranulartiy[IsNowGranulartiy["Seconds"] = 2] = "Seconds";
    IsNowGranulartiy[IsNowGranulartiy["Milliseconds"] = 3] = "Milliseconds";
})(IsNowGranulartiy || (IsNowGranulartiy = {}));
function isNow(date, granularity) {
    if (granularity === void 0) { granularity = IsNowGranulartiy.Minutes; }
    var ret = true;
    var now = new Date(Date.now());
    if (now.getFullYear() !== date.getFullYear())
        ret = false;
    if (now.getMonth() !== date.getMonth())
        ret = false;
    if (now.getDate() !== date.getDate())
        ret = false;
    if (now.getHours() !== date.getHours())
        ret = false;
    if (now.getMinutes() !== date.getMinutes() && granularity >= IsNowGranulartiy.Minutes)
        ret = false;
    if (now.getSeconds() !== date.getSeconds() && granularity >= IsNowGranulartiy.Seconds)
        ret = false;
    if (now.getMilliseconds() !== date.getMilliseconds() && granularity >= IsNowGranulartiy.Milliseconds)
        ret = false;
    return ret;
}
/***
 * Checks if the given object is a date object
 * @param obj the object to check for if it is a date or not
 */
function isDate(obj) {
    if (typeof obj !== 'object') {
        return false;
    }
    if (obj.constructor.name !== 'Date') {
        return false;
    }
    return true;
}
var DateHelper = /** @class */ (function () {
    function DateHelper() {
    }
    DateHelper.isDate = isDate;
    DateHelper.isNow = isNow;
    DateHelper.isToday = isToday;
    DateHelper.isYesterday = isYesterday;
    DateHelper.cSharpTicks2Date = cSharpTicks2Date;
    DateHelper.dateAddYears = dateAddYears;
    DateHelper.dateSubstractYears = dateSubstractYears;
    DateHelper.dateAddMonths = dateAddMonths;
    DateHelper.dateSubstractMonths = dateSubstractMonths;
    DateHelper.dateAddDays = dateAddDays;
    DateHelper.dateSubstractDays = dateSubstractDays;
    DateHelper.dateAddHours = dateAddHours;
    DateHelper.dateSubstractHours = dateSubstractHours;
    DateHelper.dateAddMinutes = dateAddMinutes;
    DateHelper.dateSubstractMinutes = dateSubstractMinutes;
    DateHelper.dateAddSeconds = dateAddSeconds;
    DateHelper.dateSubstractSeconds = dateSubstractSeconds;
    DateHelper.dateAddMilliSeconds = dateAddMilliSeconds;
    DateHelper.dateSubstractMilliSeconds = dateSubstractMilliSeconds;
    return DateHelper;
}());

/**
 * Merges 2 objects in regards that properties that are not found on the
 * mainObject are looked up on the extendedObject
 * @param mainObject Properties are first read here. if e.g. toString()
 * exists on both objects it will be returned from this one
 * @param extendedObject This is the second object that extends the mainObject.
 * Properties not found on the  mainObject are read from this one
 */
function createAutoIndexer(mainObject, extendedObject) {
    var handler = {
        get: function (obj, prop) {
            return prop in mainObject ?
                mainObject[prop] :
                extendedObject[prop];
        },
        set: function (obj, prop, value) {
            return prop in mainObject ?
                mainObject[prop] = value :
                extendedObject[prop] = value;
        }
    };
    return new Proxy(mainObject, handler);
}
/***
 * Checks if the given object has a specific constructor name
 * e.g: hasConstructorName(new Date(), 'Date')
 * @param obj the object to check for if it has the correct constructor name
 */
function hasConstructorName(obj, constructorName) {
    if (typeof obj !== 'object') {
        return false;
    }
    if (obj.constructor.name !== constructorName) {
        return false;
    }
    return true;
}
var ObjectHelper = /** @class */ (function () {
    function ObjectHelper() {
    }
    ObjectHelper.hasConstructorName = hasConstructorName;
    ObjectHelper.createAutoIndexer = createAutoIndexer;
    return ObjectHelper;
}());

function move(arr, prevIndex, newIndex) {
    while (prevIndex < 0) {
        prevIndex += arr.length;
    }
    while (newIndex < 0) {
        newIndex += arr.length;
    }
    if (newIndex >= arr.length) {
        var k = newIndex - arr.length;
        while ((k--) + 1) {
            arr.push(undefined);
        }
    }
    arr.splice(newIndex, 0, arr.splice(prevIndex, 1)[0]);
    return arr;
}
var ArrayHelper = /** @class */ (function () {
    function ArrayHelper() {
    }
    ArrayHelper.move = move;
    return ArrayHelper;
}());

/***
 * Compares 2 objects if they are equal by converting to json and comparing the
 * string So this method does not compare the reference but all the values
 * inside
 * @param obj1 First object to compare
 * @param obj2 Second object to compare
 */
function objectIsEqual(obj1, obj2) {
    var json1 = stringify(obj1);
    var json2 = stringify(obj2);
    return json1 === json2;
}
/***
 * Compares 2 dates, dates can also be undefined or null
 * if both are null or undefined they are also the same...
 * @param date1
 * @param date2
 */
function isDateEqual(date1, date2) {
    if (!date1 && !date2) {
        return true;
    }
    if (!date1 || !date2) {
        return false;
    }
    if (date1 instanceof Date && date2 instanceof Date) {
        return date1.toISOString() === date2.toISOString();
    }
    throw new Error('isDateEqual: date1 or date2 must be unset or of type Date');
}
var ComparatorHelper = /** @class */ (function () {
    function ComparatorHelper() {
    }
    ComparatorHelper.objectIsEqual = objectIsEqual;
    ComparatorHelper.isDateEqual = isDateEqual;
    return ComparatorHelper;
}());

var Helper = /** @class */ (function () {
    function Helper() {
    }
    Helper.date = DateHelper;
    Helper.form = FormHelper;
    Helper.misc = MiscHelper;
    Helper.file = FileHelper;
    Helper.string = StringHelper;
    Helper.object = ObjectHelper;
    Helper.array = ArrayHelper;
    Helper.comparator = ComparatorHelper;
    return Helper;
}());

var PubSubService = /** @class */ (function () {
    function PubSubService() {
        var _this = this;
        this.subject = new Subject();
        window.addEventListener('storage', function (event) {
            if (event.key !== PubSubService.LOCALSTORAGE_KEY)
                return;
            if (!event.newValue)
                return;
            var data = JSON.parse(event.newValue);
            _this.publish(data.topic, data.message);
        });
    }
    /**
     * Publish across browser tabs
     * @param topic Name of the topic
     * @param message The message of the topic
     * @param includeSelf True if you want to publish also to the current
     * active tab (defaults to false)
     */
    PubSubService.prototype.publishCrossTab = function (topic, message, includeSelf) {
        if (includeSelf === void 0) { includeSelf = false; }
        localStorage.setItem(PubSubService.LOCALSTORAGE_KEY, JSON.stringify({ topic: topic, message: message }));
        if (includeSelf)
            this.publish(topic, message);
    };
    /**
     * Publish a topic inside the current browser tab
     * @param topic Name of the topic
     * @param message Message of the topic
     */
    PubSubService.prototype.publish = function (topic, message) {
        this.subject.next({ topic: topic, message: message });
    };
    /**
     * Subscribe to a topics and receive messages
     * @param topics One or more topics to listen to
     */
    PubSubService.prototype.topic = function () {
        var topics = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            topics[_i] = arguments[_i];
        }
        return this.subject
            .asObservable().pipe(filter(function (data) { return topics.includes(data.topic); }), map(function (data) { return data.message; }));
    };
    PubSubService.LOCALSTORAGE_KEY = '__crossTabCommunication';
    /** @nocollapse */ PubSubService.ɵfac = function PubSubService_Factory(t) { return new (t || PubSubService)(); };
    /** @nocollapse */ PubSubService.ɵprov = ɵɵdefineInjectable({ token: PubSubService, factory: PubSubService.ɵfac, providedIn: 'root' });
    return PubSubService;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(PubSubService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return []; }, null); })();

var TemplateService = /** @class */ (function () {
    function TemplateService() {
        this.templates = new Map();
    }
    TemplateService.prototype.get = function (templateName) {
        return this.templates.get(templateName);
    };
    TemplateService.prototype.has = function (templateName) {
        return this.templates.has(templateName);
    };
    TemplateService.prototype.put = function (templateName, templateRef) {
        this.templates.set(templateName, templateRef);
    };
    /** @nocollapse */ TemplateService.ɵfac = function TemplateService_Factory(t) { return new (t || TemplateService)(); };
    /** @nocollapse */ TemplateService.ɵprov = ɵɵdefineInjectable({ token: TemplateService, factory: TemplateService.ɵfac });
    return TemplateService;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(TemplateService, [{
        type: Injectable
    }], function () { return []; }, null); })();

var CachedObservable = /** @class */ (function () {
    function CachedObservable(cacheDurationSeconds, observable) {
        this.cacheDurationSeconds = cacheDurationSeconds;
        this.observable = observable;
        this.replaySubject = new ReplaySubject(1);
        this.lastUpdatedSeconds = 0;
    }
    /***
     * Resets the cache duration, so the next attempt will fetch new
     */
    CachedObservable.prototype.reset = function () {
        this.lastUpdatedSeconds = Date.now() / 1000 - this.cacheDurationSeconds - 1;
    };
    /***
     * Refreshes (refetches) the observable
     */
    CachedObservable.prototype.refresh = function () {
        var _this = this;
        this.observable.pipe(take(1), tap(function () { return _this.lastUpdatedSeconds =
            Date.now() / 1000; }), tap(function (response) { return _this.replaySubject.next(response); })).subscribe();
    };
    CachedObservable.prototype.getObservable = function () {
        var _this = this;
        // Update the cache ?
        if (this.lastUpdatedSeconds <
            Date.now() / 1000 - this.cacheDurationSeconds) {
            return this.observable.pipe(take(1), tap(function () { return _this.lastUpdatedSeconds =
                Date.now() / 1000; }), tap(function (response) {
                _this.replaySubject.next(response);
            }));
        }
        return this.replaySubject.asObservable();
    };
    return CachedObservable;
}());

var CachedObservableService = /** @class */ (function () {
    function CachedObservableService() {
        this.dictionary = new Map();
    }
    CachedObservableService.prototype.resetAll = function () {
        this.dictionary.forEach(function (observable, key) {
            observable.reset();
        });
    };
    CachedObservableService.prototype.resetByKeyStartsWith = function (keyStartsWith) {
        this.dictionary.forEach(function (observable, key) {
            if (key.startsWith(keyStartsWith)) {
                observable.reset();
            }
        });
    };
    /***
     * Resets the cache duration, so the next attempt will fetch new
     */
    CachedObservableService.prototype.reset = function (key) {
        if (this.dictionary.has(key)) {
            this.dictionary.get(key).reset();
        }
    };
    /***
     * Refreshes (refetches) the observable
     */
    CachedObservableService.prototype.refresh = function (key) {
        if (this.dictionary.has(key)) {
            this.dictionary.get(key).refresh();
        }
    };
    CachedObservableService.prototype.getOrCreate = function (key, cacheDurationSeconds, observable) {
        if (!this.dictionary.has(key)) {
            this.dictionary.set(key, new CachedObservable(cacheDurationSeconds, observable));
        }
        return this.dictionary.get(key).getObservable();
    };
    /** @nocollapse */ CachedObservableService.ɵfac = function CachedObservableService_Factory(t) { return new (t || CachedObservableService)(); };
    /** @nocollapse */ CachedObservableService.ɵprov = ɵɵdefineInjectable({ token: CachedObservableService, factory: CachedObservableService.ɵfac, providedIn: 'root' });
    return CachedObservableService;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(CachedObservableService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return []; }, null); })();

var Services = /** @class */ (function () {
    function Services() {
    }
    Services.PubSubService = PubSubService;
    Services.TemplateService = TemplateService;
    Services.CachedObservableService = CachedObservableService;
    return Services;
}());

/**
 * This should be the very base component of every angular component
 * It uses OnInit and OnDestroy, so remember to call super.ngOnInit() at first
 * if you also use OnInit in the child class
 *
 * In nearly every angular component you will subscribe to something
 * and the best approach to unsubscribe is: takeUntil
 * takeUntil the component gets destroyed
 *
 * e.g.:
 * timer(1000).pipe(
 *  takeUntil(this.onDestroy$),
 *  tap(() => do something)
 * ).subscribe();
 *
 * so this base class has no injector dependencies and provide the onDestroy$
 * and onInit$ observables which you can subscribe to.
 *
 * e.g. you don't have to use OnInit in the subsclass you cann also just
 * subscribe to this.onInit$.subscribe() in the constructor
 * it' also useful if you want to access @Input variables which get set
 * at in onInit lifecycle and not in the constructor, so:
 *
 * @Input() someVar;
 * constructor() {
 *   this.onInit$().subscribe(() => console.log(this.someVar));
 * }
 */
var BaseComponent = /** @class */ (function () {
    function BaseComponent() {
        this.isDisabled$ = new BehaviorSubject(false);
        this.onDestroy$ubject = new ReplaySubject(1);
        this.onInit$ubject = new ReplaySubject(1);
        this.onChanges$ubject = new Subject();
        this.onDestroy$ = this.onDestroy$ubject.asObservable();
        this.onInit$ = this.onInit$ubject.asObservable();
        this.onChanges$ = this.onChanges$ubject.asObservable();
    }
    Object.defineProperty(BaseComponent.prototype, "disabled", {
        set: function (isDisabled) {
            this.isDisabled$.next(isDisabled);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Returns the SimpleChange event for the given property
     * @param property The property (name of the @Input) for which to filter
     */
    BaseComponent.prototype.onChangesByProperty$ = function (property) {
        return this.onChanges$.pipe(filter(function (ch) { return ch.property === property; }), map(function (ch) { return ch.change; }));
    };
    /**
     * Returns the currentValue of the SimpleChange event for the given property
     * @param property The property (name of the @Input) for which to filter
     */
    BaseComponent.prototype.onChangesCurrentValueByProperty$ = function (property) {
        return this.onChanges$.pipe(filter(function (ch) { return ch.property === property; }), map(function (ch) { return ch.change.currentValue; }));
    };
    BaseComponent.prototype.ngOnInit = function () {
        this.onInit$ubject.next(undefined);
        this.onInit$ubject.complete();
    };
    BaseComponent.prototype.ngOnDestroy = function () {
        this.onDestroy$ubject.next(undefined);
        this.onDestroy$ubject.complete();
        this.onChanges$ubject.complete();
    };
    BaseComponent.prototype.ngOnChanges = function (changes) {
        var _this = this;
        var changedProperties = Object.keys(changes);
        changedProperties.forEach(function (changedProperty) {
            _this.onChanges$ubject.next({
                property: changedProperty,
                change: changes[changedProperty]
            });
        });
    };
    /** @nocollapse */ BaseComponent.ɵfac = function BaseComponent_Factory(t) { return new (t || BaseComponent)(); };
    /** @nocollapse */ BaseComponent.ɵdir = ɵɵdefineDirective({ type: BaseComponent, inputs: { disabled: "disabled" }, features: [ɵɵNgOnChangesFeature] });
    return BaseComponent;
}());

/***
 * Checks if a form control date is greater than another form control date
 * @param lowerDateControl the other control which holds the date that must be lower
 * @param formValue2DateFunc if the value of the form controls is not a date provide
 * this function to convert to date
 */
function dateGreaterThan(lowerDateControl, formValue2DateFunc) {
    return function (control) {
        var lowerDate;
        var greaterDate;
        if (formValue2DateFunc) {
            lowerDate = formValue2DateFunc(lowerDateControl.value);
            greaterDate = formValue2DateFunc(control.value);
        }
        else {
            lowerDate = lowerDateControl.value;
            greaterDate = control.value;
        }
        return lowerDate > greaterDate ?
            { dateGreaterThan: { value: control.value } } : null;
    };
}
/***
 * This validator function must  be used on the formGroup level, see
 * https://angular.io/guide/form-validation#cross-field-validation
 * @param controlNames the controlNames which at least on must  have a value
 * @param hasValueFunc Optional a function to tell if the control has a value or not
 */
function anyRequired(controlNames, hasValueFunc) {
    if (hasValueFunc === void 0) { hasValueFunc = function (c) { return !!c.value; }; }
    return function (fb) {
        if (!hasConstructorName(fb, 'FormGroup')) {
            throw new Error('Validator anyRequired must be on form level. See https://angular.io/guide/form-validation#cross-field-validation');
        }
        var someHasValue = controlNames.some(function (controlName) { return hasValueFunc(fb.get(controlName)); });
        return someHasValue ? null : { anyRequired: true };
    };
}
var FormValidators = /** @class */ (function () {
    function FormValidators() {
    }
    FormValidators.anyRequired = anyRequired;
    FormValidators.dateGreaterThan = dateGreaterThan;
    return FormValidators;
}());

var RoundNumberPipe = /** @class */ (function () {
    function RoundNumberPipe() {
    }
    RoundNumberPipe.prototype.transform = function (value, digits) {
        return roundNumber(value, digits);
    };
    /** @nocollapse */ RoundNumberPipe.ɵfac = function RoundNumberPipe_Factory(t) { return new (t || RoundNumberPipe)(); };
    /** @nocollapse */ RoundNumberPipe.ɵpipe = ɵɵdefinePipe({ name: "roundNumber", type: RoundNumberPipe, pure: true });
    return RoundNumberPipe;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(RoundNumberPipe, [{
        type: Pipe,
        args: [{ name: 'roundNumber' }]
    }], null, null); })();

var NgIonicSyncFormcontrolClassesDirective = /** @class */ (function () {
    function NgIonicSyncFormcontrolClassesDirective(el) {
        var _this = this;
        this.el = el;
        this.ionDirty = false;
        this.ionPristine = true;
        this.ionTouched = false;
        this.ionUnTouched = true;
        var setIonItemClass = function (cssClass, visible) {
            if (!_this.ionItem) {
                return;
            }
            if (!visible) {
                _this.ionItem.classList.remove(cssClass);
            }
            else if (!_this.ionItem.className.includes(cssClass)) {
                _this.ionItem.classList.add(cssClass);
            }
        };
        var checkMutations = new Subject();
        checkMutations.pipe(debounceTime(100), tap(function () {
            var cssClasses = el.nativeElement.className;
            if (cssClasses.includes('ng-touched')) {
                _this.ionTouched = true;
                _this.ionUnTouched = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-touched', true);
                setIonItemClass('ion-untouched', false);
            }
            if (cssClasses.includes('ng-untouched')) {
                _this.ionUnTouched = true;
                _this.ionTouched = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-touched', false);
                setIonItemClass('ion-untouched', true);
            }
            if (cssClasses.includes('ng-pristine')) {
                _this.ionPristine = true;
                _this.ionDirty = false;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-pristine', true);
                setIonItemClass('ion-dirty', false);
            }
            if (cssClasses.includes('ng-dirty')) {
                _this.ionPristine = false;
                _this.ionDirty = true;
                // If we have an ion-item as a parent also sync its classes
                setIonItemClass('ion-pristine', false);
                setIonItemClass('ion-dirty', true);
            }
        })).subscribe();
        this.mutationObserver = new MutationObserver(function (mutations) {
            // Use a subject for debouncing
            checkMutations.next(mutations);
        });
        this.mutationObserver.observe(el.nativeElement, {
            attributes: true,
            attributeFilter: ['style', 'class']
        });
    }
    Object.defineProperty(NgIonicSyncFormcontrolClassesDirective.prototype, "ionItem", {
        get: function () {
            if (!this._ionItem) {
                this._ionItem = this.el.nativeElement.closest('ion-item');
            }
            return this._ionItem;
        },
        enumerable: true,
        configurable: true
    });
    NgIonicSyncFormcontrolClassesDirective.prototype.ngOnDestroy = function () {
        this.mutationObserver.disconnect();
    };
    /** @nocollapse */ NgIonicSyncFormcontrolClassesDirective.ɵfac = function NgIonicSyncFormcontrolClassesDirective_Factory(t) { return new (t || NgIonicSyncFormcontrolClassesDirective)(ɵɵdirectiveInject(ElementRef)); };
    /** @nocollapse */ NgIonicSyncFormcontrolClassesDirective.ɵdir = ɵɵdefineDirective({ type: NgIonicSyncFormcontrolClassesDirective, selectors: [["", "ngIonSync", ""]], hostVars: 8, hostBindings: function NgIonicSyncFormcontrolClassesDirective_HostBindings(rf, ctx) { if (rf & 2) {
            ɵɵclassProp("ion-dirty", ctx.ionDirty)("ion-pristine", ctx.ionPristine)("ion-touched", ctx.ionTouched)("ion-untouched", ctx.ionUnTouched);
        } } });
    return NgIonicSyncFormcontrolClassesDirective;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(NgIonicSyncFormcontrolClassesDirective, [{
        type: Directive,
        args: [{
                // tslint:disable-next-line:directive-selector
                selector: '[ngIonSync]'
            }]
    }], function () { return [{ type: ElementRef }]; }, { ionDirty: [{
            type: HostBinding,
            args: ['class.ion-dirty']
        }], ionPristine: [{
            type: HostBinding,
            args: ['class.ion-pristine']
        }], ionTouched: [{
            type: HostBinding,
            args: ['class.ion-touched']
        }], ionUnTouched: [{
            type: HostBinding,
            args: ['class.ion-untouched']
        }] }); })();

var ShadowDomStyleDirective = /** @class */ (function () {
    function ShadowDomStyleDirective(el) {
        this.el = el;
    }
    Object.defineProperty(ShadowDomStyleDirective.prototype, "shadowStyle", {
        set: function (value) {
            var elem = this.el.nativeElement;
            waitForProperty(elem, function (obj) { return elem.shadowRoot.querySelector(value.selector); })
                .subscribe(function (shadowElem) {
                var e_1, _a;
                if (!shadowElem || !value) {
                    return;
                }
                try {
                    for (var _b = __values(Object.keys(value.style)), _c = _b.next(); !_c.done; _c = _b.next()) {
                        var style = _c.value;
                        if (!shadowElem.style || !value.style) {
                            continue;
                        }
                        shadowElem.style[style] = value.style[style];
                    }
                }
                catch (e_1_1) { e_1 = { error: e_1_1 }; }
                finally {
                    try {
                        if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                    }
                    finally { if (e_1) throw e_1.error; }
                }
            });
        },
        enumerable: true,
        configurable: true
    });
    /** @nocollapse */ ShadowDomStyleDirective.ɵfac = function ShadowDomStyleDirective_Factory(t) { return new (t || ShadowDomStyleDirective)(ɵɵdirectiveInject(ElementRef)); };
    /** @nocollapse */ ShadowDomStyleDirective.ɵdir = ɵɵdefineDirective({ type: ShadowDomStyleDirective, selectors: [["", "shadowStyle", ""]], inputs: { shadowStyle: "shadowStyle" } });
    return ShadowDomStyleDirective;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(ShadowDomStyleDirective, [{
        type: Directive,
        args: [{
                selector: '[shadowStyle]'
            }]
    }], function () { return [{ type: ElementRef }]; }, { shadowStyle: [{
            type: Input
        }] }); })();

var SwissArmyKnifeModule = /** @class */ (function () {
    function SwissArmyKnifeModule() {
    }
    /** @nocollapse */ SwissArmyKnifeModule.ɵmod = ɵɵdefineNgModule({ type: SwissArmyKnifeModule });
    /** @nocollapse */ SwissArmyKnifeModule.ɵinj = ɵɵdefineInjector({ factory: function SwissArmyKnifeModule_Factory(t) { return new (t || SwissArmyKnifeModule)(); }, imports: [[]] });
    return SwissArmyKnifeModule;
}());
(function () { (typeof ngJitMode === "undefined" || ngJitMode) && ɵɵsetNgModuleScope(SwissArmyKnifeModule, { declarations: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective], exports: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective] }); })();
/*@__PURE__*/ (function () { ɵsetClassMetadata(SwissArmyKnifeModule, [{
        type: NgModule,
        args: [{
                declarations: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective],
                imports: [],
                exports: [RoundNumberPipe, NgIonicSyncFormcontrolClassesDirective, ShadowDomStyleDirective]
            }]
    }], null, null); })();

var KeyValueStoreInitialValue = new InjectionToken('KeyValueStoreInitialValue');
var KeyValueStoreService = /** @class */ (function () {
    function KeyValueStoreService(initialStore) {
        var _this = this;
        this.initialStore = initialStore;
        this.LOCALSTORAGE_KEY = 'KeyValueStore';
        this.changes$ = new Subject();
        var currentValue = this.getFromLocalStorage(this.LOCALSTORAGE_KEY);
        this.store$ = new BehaviorSubject((currentValue || initialStore || {}));
        this.changes$.subscribe(function (change) {
            var store = _this.store$.getValue();
            store[change.key] = change.currentValue;
            _this.store$.next(store);
        });
        this.store$.subscribe(function (store) {
            localStorage.setItem(_this.LOCALSTORAGE_KEY, JSON.stringify(store));
        });
    }
    KeyValueStoreService.prototype.getStore = function () {
        return this.store$.getValue();
    };
    KeyValueStoreService.prototype.getStore$ = function () {
        return this.store$.asObservable();
    };
    KeyValueStoreService.prototype.getAllChanges$ = function () {
        return this.changes$.asObservable();
    };
    KeyValueStoreService.prototype.resetStore = function () {
        this.store$ = new BehaviorSubject((this.initialStore || {}));
    };
    KeyValueStoreService.prototype.getValue$ = function (key, startWithCurrentValue) {
        if (startWithCurrentValue === void 0) { startWithCurrentValue = true; }
        return startWithCurrentValue ?
            concat(of(this.getValue(key)), this.changes$.asObservable().pipe(filter(function (change) { return change.key === key; }), map(function (change) { return change.currentValue; })))
            : this.changes$.asObservable().pipe(filter(function (change) { return change.key === key; }), map(function (change) { return change.currentValue; }));
    };
    KeyValueStoreService.prototype.getValueChanges$ = function (key) {
        return this.changes$.asObservable().pipe(filter(function (change) { return change.key === key; }));
    };
    KeyValueStoreService.prototype.getFromLocalStorage = function (key) {
        try {
            return JSON.parse(localStorage.getItem(key));
        }
        catch (_a) {
            return undefined;
        }
    };
    KeyValueStoreService.prototype.getValue = function (key) {
        return this.getStore()[key];
    };
    KeyValueStoreService.prototype.setValue = function (key, value) {
        this.changes$.next({
            key: key,
            currentValue: value,
            previousValue: this.getFromLocalStorage(key)
        });
    };
    /** @nocollapse */ KeyValueStoreService.ɵfac = function KeyValueStoreService_Factory(t) { return new (t || KeyValueStoreService)(ɵɵinject(KeyValueStoreInitialValue, 8)); };
    /** @nocollapse */ KeyValueStoreService.ɵprov = ɵɵdefineInjectable({ token: KeyValueStoreService, factory: KeyValueStoreService.ɵfac, providedIn: 'root' });
    return KeyValueStoreService;
}());
/*@__PURE__*/ (function () { ɵsetClassMetadata(KeyValueStoreService, [{
        type: Injectable,
        args: [{
                providedIn: 'root'
            }]
    }], function () { return [{ type: undefined, decorators: [{
                type: Inject,
                args: [KeyValueStoreInitialValue]
            }, {
                type: Optional
            }] }]; }, null); })();

var StatefulObject = /** @class */ (function () {
    function StatefulObject() {
        var _this = this;
        this.stateSubject = new BehaviorSubject({});
        this.state$ = this.stateSubject.asObservable();
        this.stateSubject.subscribe(function (s) { return _this.stateSnapshot = s; });
    }
    StatefulObject.prototype.setState = function (newState) {
        this.stateSubject.next(__assign(__assign({}, this.stateSubject.getValue()), newState));
    };
    return StatefulObject;
}());

/*
 * Public API Surface of ng-swiss-army-knife
 */
var SwissArmyKnife = /** @class */ (function () {
    function SwissArmyKnife() {
    }
    SwissArmyKnife.FormValidators = FormValidators;
    SwissArmyKnife.Helper = Helper;
    SwissArmyKnife.Services = Services;
    SwissArmyKnife.BaseComponent = BaseComponent;
    return SwissArmyKnife;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { ArrayHelper, BaseComponent, CachedObservable, CachedObservableService, ComparatorHelper, DateHelper, FileHelper, FormHelper, FormValidators, Helper, IsNowGranulartiy, KeyValueStoreInitialValue, KeyValueStoreService, MiscHelper, NgIonicSyncFormcontrolClassesDirective, ObjectHelper, PubSubService, RoundNumberPipe, ShadowDomStyleDirective, StatefulObject, StringHelper, SwissArmyKnife, SwissArmyKnifeModule, TemplateService, anyRequired, cSharpTicks2Date, createAndClickDownloadLink, createAutoIndexer, createCsvDataUriFromArray, createThumbnailFromBase64Image, dateAddDays, dateAddHours, dateAddMilliSeconds, dateAddMinutes, dateAddMonths, dateAddSeconds, dateAddYears, dateGreaterThan, dateSubstractDays, dateSubstractHours, dateSubstractMilliSeconds, dateSubstractMinutes, dateSubstractMonths, dateSubstractSeconds, dateSubstractYears, decodeURIJson, downloadArrayAsCsvFile, encodeURIJson, escapeXml, getDirtyControls, getFile, hasConstructorName, isDate, isDateEqual, isNow, isNullOrUndefined, isNumber, isObjectEmpty, isToday, isYesterday, jsonDeepClone, jsonIsEqual, loadImage, markFormGroupAsDirty, markFormGroupAsPending, markFormGroupAsPristine, markFormGroupAsTouched, markFormGroupAsUntouched, move, objectIsEqual, padLeft, padRight, removeBodyCssClass, replaceAll, roundNumber, safeCall, setBodyCssClass, toggleBodyCssClass, waitForProperty };
//# sourceMappingURL=ng-swiss-army-knife.js.map
