/**
 * Creates a hidden link to download something given the href url
 * @param url The url which to download
 * @param fileName The name of the file to be downloaded (used for the a tag)
 */
export declare function createAndClickDownloadLink(url: string, fileName: string): void;
/**
 * Creates a csv data uri from an array, this uri can then be downloaded in the browser
 * using createAndClickDownloadLink
 * @param data an array of string arrays containing the csv data
 */
export declare function createCsvDataUriFromArray(data: string[][], delimiter?: string): string;
/**
 * Creates a csv file from the array using createCsvDataUriFromArray
 * And presents it to the user as a download file in the browser
 * @param data an array of string arrays containing the csv data
 * @param filename the name of the csv file which will be downloaded
 */
export declare function downloadArrayAsCsvFile(data: string[][], filename: string, delimiter?: string): void;
export declare type FileDataType = 'base64' | 'binaryString';
export interface IFile {
    data: string;
    dataType: FileDataType;
    lastModified: number;
    name: string;
    size: number;
    mimeType: string;
}
/**
 * opens a file dialog to choose a file and returns the file object
 * including the data.
 * When it is an image the data is read as base64, so the dataType is 'base64'
 * All other files are read as binaryString
 * @param fileDialogAccept this is optional, choose the mime types of files
 * the user can select in the file dialog
 */
export declare function getFile(fileDialogAccept?: string): Promise<IFile>;
export declare class FileHelper {
    static getFile: typeof getFile;
    static createAndClickDownloadLink: typeof createAndClickDownloadLink;
    static createCsvDataUriFromArray: typeof createCsvDataUriFromArray;
    static downloadArrayAsCsvFile: typeof downloadArrayAsCsvFile;
}
